#ifndef LWIPOPTS_H
#define LWIPOPTS_H

/* This file is included from *only* 1 place:
 *  $(TOP_DIR)/src/include/lwip/opt.h.
 *
 * It's included both while building (LWIP_BUILDING=1) and while using LwIP
 * (LWIP_BUILDING=0).Take great care while tuning the values below. There are
 * plenty of them....
 */
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#define SYS_ARCH_PROTECT(lev)         ((void)0)   /* we don't care about protection */
#define SYS_ARCH_DECL_PROTECT(lev)    ((void)0)
#define SYS_ARCH_UNPROTECT(lev)       ((void)0)

#define NO_SYS                        1
#define USE_LOOPBACK                  1

#define SYS_DEBUG                     1

#if (NO_SYS == 0)
  #define sys_sem_t    unsigned char  /* does not matter */
  #define sys_mbox_t   unsigned char  /* does not matter */
  #define sys_thread_t unsigned char  /* does not matter */

  #define LWIP_COMPAT_MUTEX_ALLOWED

  #define sys_arch_protect()         ((void) 0)
  #define sys_arch_unprotect(x)      ((void) 0)
#endif

/* MbedTLS on MSDOS! Possible?
 */
#if defined(LWIP_USE_MBEDTLS)
  #define LWIP_ALTCP                  1
  #define LWIP_ALTCP_TLS              1
  #define LWIP_ALTCP_TLS_MBEDTLS      1
#endif

#define LWIP_SOCKET                   (NO_SYS==0)
#define LWIP_NETCONN                  (NO_SYS==0)
#define LWIP_DEBUG                    1
#define LWIP_DEBUG_TIMERNAMES         1
#define LWIP_IGMP                     1
#define LWIP_ICMP                     1
#define LWIP_SNMP                     1
#define LWIP_DHCP                     1
#define LWIP_DNS                      1
#define LWIP_IPV4                     1
#define LWIP_IPV6                     1
#define LWIP_IPV6_FRAG                0
#define LWIP_IPV6_REASS               1
#define LWIP_NETIF_TX_SINGLE_PBUF     0
#define LWIP_HAVE_LOOPIF              USE_LOOPBACK
#define LWIP_NETIF_LOOPBACK           USE_LOOPBACK
#define LWIP_LOOPBACK_MAX_PBUFS       10
#define LWIP_NETIF_LINK_CALLBACK      1
#define LWIP_NETIF_STATUS_CALLBACK    1
#define LWIP_NETIF_API                (NO_SYS==0)
#define LWIP_NETIF_HOSTNAME           1
#define LWIP_COMPAT_SOCKETS           1
#define LWIP_SO_RCVTIMEO              1
#define LWIP_SO_RCVBUF                1
#define LWIP_COMPAT_MUTEX             1

#define TCP_LISTEN_BACKLOG            5
#define TCP_MSS                       1460
#define TCP_NUM_IN_SND_BUF            10
#define TCP_SND_BUF                   (TCP_NUM_IN_SND_BUF*TCP_MSS)
#define TCP_WND                       (TCP_NUM_IN_SND_BUF*TCP_MSS)

#define ARP_QUEUEING                  1
#define ETHARP_TRUST_IP_MAC           1
#define ETHARP_SUPPORT_STATIC_ENTRIES 1
#define UDP_TTL                       255

#define DNS_SERVER_ADDRESS(addr)   \
        ip_addr_set_ip4_u32(addr, ipaddr_addr("8.8.8.8")) /* google-public-dns-a.google.com */

#define MEM_LIBC_MALLOC               1
#define MEMP_MEM_MALLOC               1

#ifdef LWIP_DEBUG
  #define LWIP_DBG_MIN_LEVEL          0
  #define MEMP_OVERFLOW_CHECK         2

  #define PKT_IF_DEBUG                1
  #define PPP_DEBUG                   1
  #define MEM_DEBUG                   1
  #define MEMP_DEBUG                  1
  #define MEMP_SANITY_CHECK           1

  #define PBUF_DEBUG                  1
  #define API_LIB_DEBUG               1
  #define API_MSG_DEBUG               1
  #define TCPIP_DEBUG                 1
  #define NETIF_DEBUG                 1
  #define ETHARP_DEBUG                1
  #define SOCKETS_DEBUG               1
  #define DNS_DEBUG                   1
  #define AUTOIP_DEBUG                1
  #define DHCP_DEBUG                  1
  #define IP_DEBUG                    1
  #define IP_REASS_DEBUG              1
  #define ICMP_DEBUG                  1
  #define IGMP_DEBUG                  1
  #define UDP_DEBUG                   1
  #define TCP_DEBUG                   1
  #define TCP_INPUT_DEBUG             1
  #define TCP_OUTPUT_DEBUG            1
  #define TCP_RTO_DEBUG               1
  #define TCP_CWND_DEBUG              1
  #define TCP_WND_DEBUG               1
  #define TCP_FR_DEBUG                1
  #define TCP_QLEN_DEBUG              1
  #define TCP_RST_DEBUG               1
  #define INET_DEBUG                  1
  #define RAW_DEBUG                   1
  #define RTP_DEBUG                   0
  #define TIMERS_DEBUG                1
  #define SNMP_MSG_DEBUG              1
  #define SNMP_MIB_DEBUG              1
  #define IP6_DEBUG                   1
#endif   /* LWIP_DEBUG > 0 */

#if (NO_SYS == 0)
  #ifdef __DJGPP__
  #define sys_msleep(msec)    usleep(1000*(msec))
  #endif
#endif

extern unsigned int lwip_debug_flags;

#define LWIP_DBG_TYPES_ON lwip_debug_flags

/* ALMOST EVERYTHING BELOW IS A COPY OF "lwip/opt.h". Hence it's not
 * getting redifined in "lwip/opt.h". But defined here so it can easily
 * be changed in one place.
 */

/* MEMP_NUM_TCP_SEG: the number of simultaneously queued TCP segments.
 * (requires the LWIP_TCP option)
 */
#define MEMP_NUM_TCP_SEG        TCP_SND_QUEUELEN

/* MEMP_NUM_SYS_TIMEOUT: the number of simulateously active
   timeouts. */
#define MEMP_NUM_SYS_TIMEOUT    10

/* The following four are used only with the sequential API and can be
   set to 0 if the application only will use the raw API. */
/* MEMP_NUM_NETBUF: the number of struct netbufs. */
#define MEMP_NUM_NETBUF         10

/* MEMP_NUM_NETCONN: the number of struct netconns. */
#define MEMP_NUM_NETCONN        10

/* MEMP_NUM_TCPIP_MSG_*: the number of struct tcpip_msg, which is used
   for sequential API communication and incoming packets. Used in
   src/api/tcpip.c. */
#define MEMP_NUM_TCPIP_MSG_API   16
#define MEMP_NUM_TCPIP_MSG_INPKT 16

#if 1
  /* ---------- Pbuf options ---------- */
  /* PBUF_POOL_SIZE: the number of buffers in the pbuf pool. */
  #define PBUF_POOL_SIZE          500

  /* PBUF_POOL_BUFSIZE: the size of each pbuf in the pbuf pool. */
  #define PBUF_POOL_BUFSIZE       128

  /* PBUF_LINK_HLEN: the number of bytes that should be allocated for a
     link level header. */
  #define PBUF_LINK_HLEN          16
#endif

/** SYS_LIGHTWEIGHT_PROT
 * define SYS_LIGHTWEIGHT_PROT in lwipopts.h if you want inter-task protection
 * for certain critical regions during buffer allocation, deallocation and memory
 * allocation and deallocation.
 */
#define SYS_LIGHTWEIGHT_PROT    1

/* ---------- TCP options ---------- */
#define LWIP_TCP                1
#define TCP_TTL                 255

/* Controls if TCP should queue segments that arrive out of
   order. Define to 0 if your device is low on memory. */
#define TCP_QUEUE_OOSEQ         1

/* TCP sender buffer space (pbufs). This must be at least = 4 *
 * TCP_NUM_IN_SND_BUF for things to work. = 40
 */
#define TCP_SND_QUEUELEN       (4 * TCP_NUM_IN_SND_BUF)

/* TCP writable space (bytes). This must be less than or equal
   to TCP_SND_BUF. It is the amount of space which must be
   available in the tcp snd_buf for select to return writable */
#define TCP_SNDLOWAT           (TCP_SND_BUF/2)

/* Maximum number of retransmissions of data segments. */
#define TCP_MAXRTX              12

/* Maximum number of retransmissions of SYN segments. */
#define TCP_SYNMAXRTX           4

/* ---------- ARP options ---------- */
#define LWIP_ARP                1
#define ARP_TABLE_SIZE          10
#define ARP_QUEUEING            1

/* ---------- IP options ---------- */
/* Define IP_FORWARD to 1 if you wish to have the ability to forward
   IP packets across network interfaces. If you are going to run lwIP
   on a device with only one network interface, define this to 0. */
#define IP_FORWARD              0

/* IP reassembly and segmentation. These are orthogonal even
 * if they both deal with IP fragments */
#define IP_REASSEMBLY           1
#define IP_REASS_MAX_PBUFS      10
#define MEMP_NUM_REASSDATA      10
#define IP_FRAG                 1

/* ---------- ICMP options ---------- */
#define ICMP_TTL                255

/* 1 if you want to do an ARP check on the offered address
   (recommended). */
#define DHCP_DOES_ARP_CHECK    (LWIP_DHCP)

/* ---------- AUTOIP options ------- */
#define LWIP_AUTOIP             0
#define LWIP_DHCP_AUTOIP_COOP  (LWIP_DHCP && LWIP_AUTOIP)

/* ---------- UDP options ---------- */
#define LWIP_UDP                1
#define LWIP_UDPLITE            1
#define UDP_TTL                 255

/* ---------- Statistics options ---------- */

#define LWIP_STATS              1
#define LWIP_STATS_LARGE        1
#define LWIP_STATS_DISPLAY      1

#if LWIP_STATS
  #define LINK_STATS            1
  #define IP_STATS              1
  #define ICMP_STATS            1
  #define IGMP_STATS            1
  #define IPFRAG_STATS          1
  #define UDP_STATS             1
  #define TCP_STATS             1
  #define MEM_STATS             1
  #define MEMP_STATS            1
  #define PBUF_STATS            1
  #define SYS_STATS             1
  #define MIB2_STATS            1
#endif /* LWIP_STATS */

/* ---------- PPP options ---------- */

#define PPP_SUPPORT             0      /* Set > 0 for PPP */

#if PPP_SUPPORT
  #define NUM_PPP               1      /* Max PPP sessions. */

  /* Select modules to enable.  Ideally these would be set in the makefile but
   * we're limited by the command line length so you need to modify the settings
   * in this file.
   */
  #define PPPOE_SUPPORT         1
  #define PPPOS_SUPPORT         1

  #define PAP_SUPPORT           1      /* Set > 0 for PAP. */
  #define CHAP_SUPPORT          1      /* Set > 0 for CHAP. */
  #define MSCHAP_SUPPORT        0      /* Set > 0 for MSCHAP (NOT FUNCTIONAL!) */
  #define CBCP_SUPPORT          0      /* Set > 0 for CBCP (NOT FUNCTIONAL!) */
  #define CCP_SUPPORT           0      /* Set > 0 for CCP (NOT FUNCTIONAL!) */
  #define VJ_SUPPORT            1      /* Set > 0 for VJ header compression. */
  #define MD5_SUPPORT           1      /* Set > 0 for MD5 (see also CHAP) */
#endif /* PPP_SUPPORT */

/*
 * What 'src/contrib/apps' or 'contrib/apps' to include:
 */
#define LWIP_DNS_APP               1
#define LWIP_CHARGEN_APP           0
#define LWIP_PING_APP              1
#define LWIP_NETBIOS_APP           1
#define LWIP_NETIO_APP             0
#define LWIP_HTTPD_APP             1
#define HTTPD_DEBUG                1
#define LWIP_HTTPD_SSI             1
#define LWIP_HTTPD_SUPPORT_POST    0
#define LWIP_HTTPD_APP_NETCONN     (NO_SYS==0)
#define LWIP_RTP_APP               0
#define LWIP_SNTP_APP              1
#define LWIP_SHELL_APP             1
#define LWIP_TCPECHO_APP           1
#define LWIP_TCPECHO_APP_NETCONN   0
#define LWIP_SOCKET_EXAMPLES_APP   1
#define LWIP_UDPECHO_APP           1
#define SOCK_TARGET_HOST4          "10.0.0.1"         /* My router/GW */

/*
 * Defines and options for 'src/apps/sntp'
 */
#define SNTP_DEBUG                    1
#define SNTP_SOCKET                   0
#define SNTP_SERVER_DNS               1
#define SNTP_CHECK_RESPONSE           2
#define SNTP_SET_SYSTEM_TIME(sec)     printf ("setting sys-time: %lu\n", (u32_t)(sec))
#define SNTP_SERVER_ADDRESS           "time.nist.gov"
#define SNTP_MAX_SERVERS              1

/*
 * Defines and options for 'src/apps/smtp'
 */
#define SMTP_DEBUG  1

/*
 * Defines and options for 'src/contrib/apps/rtp'
 */
#define RTP_DEBUG                     0
#define RTP_RECV_PROCESSING(p,s)

/*
 * Defines and options for 'src/contrib/apps/ping'
 */
#define PING_USE_SOCKETS    (NO_SYS==0)
#define PING_DEBUG          1
#define PING_DELAY          2000
#define PING_RESULT(ok)     (void)0  // printf ("PING_RESULT: %d\n", ok);

/*
 * Defines and options for 'src/apps/netbiosNS'
 */
#define NETBIOS_LWIP_NAME   "SNURRE"

/* configuration for PPP (untested)
 */
#define PPP_USERNAME   "Admin"
#define PPP_PASSWORD   "pass"

/* It's vital that we free the pktdrvr handle before exiting.
 */
extern int PktExitDriver (void);

#if defined(LWIP_BUILDING)
  #define LWIP_EXIT_FUNCS()  PktExitDriver()
#else
  #define LWIP_EXIT_FUNCS()  ((void)0)
#endif

#ifdef LWIP_NO_ABORT
  #define LWIP_ABORT()  ((void)0)
#else
  #define LWIP_ABORT()  abort()
#endif

/* Plaform specific diagnostics output
 */
#define LWIP_PLATFORM_DIAG(x)   do { printf x; } while (0)

#undef  LWIP_PLATFORM_ASSERT
#define LWIP_PLATFORM_ASSERT(x) do {                                            \
                                  printf ("Assertion \"%s\" failed at %s/%u\n", \
                                          x, __FILE__, __LINE__);               \
                                  fflush (NULL);                                \
                                  LWIP_EXIT_FUNCS();                            \
                                  LWIP_ABORT();                                 \
                                } while (0)

#define LWIP_ERROR(message, expression, handler)                                   \
                                do {                                               \
                                  if (!(expression)) {                             \
                                     printf ("Assertion \"%s\" failed at %s/%u\n", \
                                             message, __FILE__, __LINE__);         \
                                     fflush (NULL);                                \
                                     LWIP_EXIT_FUNCS();                            \
                                     LWIP_ABORT();                                 \
                                     handler;                                      \
                                  }                                                \
                                } while (0)

/*
 * The LWIP_TRACE() macro is used for internal tracing.
 */
#include <contrib/ports/win32/cfg_file.h>

/*
 * I'm tired of the messy macro in <lwip/debug.h>; I'm including that file
 * here in order to redefine that ugly LWIP_DEBUGF() macro therein.
 */
#include <lwip/debug.h>

#undef  LWIP_DEBUGF
#define LWIP_DEBUGF(debug, message)       \
        do {                              \
          if ((debug) & lwip_debug_flags) \
             printf message;              \
        } while (0)

#undef  LWIP_ASSERT
#define LWIP_ASSERT(message, assertion)      \
        do {                                 \
          if (!(assertion))                  \
             LWIP_PLATFORM_ASSERT (message); \
        } while (0)

#endif  /* LWIPOPTS_H */
